#include "StdAfx.h"
#include "xtmpwnd.h"

IMPLEMENT_DYNAMIC(XTmpWnd, CWnd)
XTmpWnd::XTmpWnd(void)
{
	m_Direction = MD_HORIZONTAL;
	m_ItemsNumber = 0;
	m_ItemWidth = 150; 
	m_ItemHeight = 30;

	m_ItemTitles = 0;

	m_X0 = m_Y0 = 0;

    m_DefaultColor = 0xc0c0c0;
    m_ActiveColor = 0xe77e7e;
	m_SelectedColor = 0x0000ff;
	m_TextColor = 0;
	m_CurrentTitle = 0;
	
	VERIFY(m_Font.CreateFont(
			20,                        // nHeight
			0,                         // nWidth
			0,                         // nEscapement
			0,                         // nOrientation
			FW_NORMAL,                 // nWeight
			TRUE,                     // bItalic
			FALSE,                     // bUnderline
			0,                         // cStrikeOut
			DEFAULT_CHARSET,              // nCharSet
			OUT_DEFAULT_PRECIS,        // nOutPrecision
			CLIP_DEFAULT_PRECIS,       // nClipPrecision
			DEFAULT_QUALITY,           // nQuality
			DEFAULT_PITCH | FF_SWISS,  // nPitchAndFamily
			"Times"));                 // lpszFacename
}

XTmpWnd::~XTmpWnd(void)
{
	ClearItemTitles();
	m_Font.DeleteObject(); 
}

void XTmpWnd::SetCoordinates(int x, int y)
{
	m_X0 = x;
	m_Y0 = y;
}

RECT XTmpWnd::GetMenuRect(void)
{
	RECT r;
	r.left	= m_X0;
	r.top	= m_Y0;
	if (m_Direction == MD_HORIZONTAL)
	{
		r.right = r.left + m_ItemsNumber * m_ItemWidth;
		r.bottom = r.top + m_ItemHeight;
	}
	else if (m_Direction == MD_VERTICAL)
	{
		r.right = r.left + m_ItemWidth;
		r.bottom = r.top + m_ItemsNumber * m_ItemHeight;
	}
	return r;
}

void XTmpWnd::GetSubMenuCoordinates(int &x, int &y, int num)
{
	if (m_Direction == MD_HORIZONTAL)
	{
		x = m_X0 + num * m_ItemWidth;
		y = m_Y0 + m_ItemHeight; 
	}
	else if (m_Direction == MD_VERTICAL)
	{
		x = m_X0 + m_ItemWidth;
		y = m_Y0 + num * m_ItemHeight; 
	}
}


void XTmpWnd::DrawDefaultItem(CPaintDC* dc, int num)
{
	DrawItem(dc, m_DefaultColor, num);
}

void XTmpWnd::DrawActiveItem(CPaintDC* dc, int num)
{
	DrawItem(dc, m_ActiveColor, num);
}

void XTmpWnd::DrawSelectedItem(CPaintDC* dc, int num)
{
	DrawItem(dc, m_SelectedColor, num);
}

void XTmpWnd::DrawItem(CPaintDC* dc, COLORREF c, int num)
{
	int x0, y0, x1, y1;
	if (m_Direction == MD_HORIZONTAL)
	{
		x0 = m_ItemWidth * num;
		y0 = 0;
	}
	else
	{
		x0 = 0;
		y0 = m_ItemHeight * num;
	}
	x1 = x0 + m_ItemWidth;
	y1 = y0 + m_ItemHeight;

	dc->FillSolidRect(x0, y0, m_ItemWidth, m_ItemHeight, c);

	RECT r;
	r.left = x0;
	r.top = y0;
	r.right = x1;
	r.bottom = y1;

	dc->DrawText(m_ItemTitles[num], -1, &r, DT_SINGLELINE | DT_VCENTER | DT_CENTER);
}

bool XTmpWnd::AddItemTitle(char* title)
{
	int len = strlen(title);
	if (m_CurrentTitle < (m_ItemsNumber))
	{
		m_ItemTitles[m_CurrentTitle] = new char[len + 1];
        strcpy(m_ItemTitles[m_CurrentTitle], title);
		m_CurrentTitle++;
		return 1;
	}
	return 0;
}

bool XTmpWnd::InsertItemTitle(char* title, int num)
{
	int len = strlen(title);
	if (num < (m_ItemsNumber))
	{
		if (m_ItemTitles[num])
		{
			delete [] m_ItemTitles[num];
		}
		m_ItemTitles[num] = new char[len + 1];
        strcpy(m_ItemTitles[num], title);
		return 1;
	}
	return 0;
}

void XTmpWnd::Show()
{
	if (m_Direction == MD_HORIZONTAL)
	{
		MoveWindow(m_X0, m_Y0, m_ItemsNumber * m_ItemWidth, m_ItemHeight);
	}
	else if (m_Direction == MD_VERTICAL)
	{
		MoveWindow(m_X0, m_Y0, m_ItemWidth, m_ItemsNumber * m_ItemHeight);
	}
}

void XTmpWnd::Hide()
{
	MoveWindow(m_X0, m_Y0, 0, 0);
}

void XTmpWnd::InitItemTitles(int num)
{
	m_ItemsNumber = num;
	m_ItemTitles = new char*[m_ItemsNumber];
	for (int i = 0; i < m_ItemsNumber; i++)
	{
		m_ItemTitles[i] = 0;
	}
}

void XTmpWnd::ClearItemTitles()
{
	if (m_ItemTitles)
	{
		for (int i = 0; i < m_ItemsNumber; i++)
		{
			delete [] m_ItemTitles[i];
		}
		delete [] m_ItemTitles;
	}
	m_ItemsNumber = 0;
	m_CurrentTitle = 0;
}

BEGIN_MESSAGE_MAP(XTmpWnd, CWnd)
	
END_MESSAGE_MAP()