#include "automat.h"
#include <memory.h>

/*----------------------------------[ XEvent ]---------------------------*/

XEvent::XEvent()
{
	m_Info = new char[4];
	for (int i = 0; i < 4; i++)
	{
		((char*)m_Info)[i] = 0;
	}
}

XEvent::XEvent(int n)
{
	m_Info = new char[n];
	for (int i = 0; i < n; i++)
	{
		((char*)m_Info)[i] = 0;
	}
}

XEvent::~XEvent()
{
	delete [] m_Info;
}

/*----------------------------------[ XState ]---------------------------*/

XState::XState()
{
	m_Title = 0;
	SetTitle("Untitled");
}

XState::XState(const char* title)
{
	m_Title = 0;
	SetTitle(title);
}

void XState::SetTitle(const char* title)
{
	if (m_Title)
	{
		delete [] m_Title;
	}
	int len = strlen(title);
	m_Title = new char[len + 1];
	strcpy(m_Title, title);
}

char* XState::GetTitle()
{
	return m_Title;
}

XState::~XState()
{
	delete [] m_Title;
}
/*----------------------------------[ XAutomat ]---------------------------*/

XAutomat::XAutomat(void)
{
	m_BeginLogging = m_TransLogging = m_EndLogging = 0;
	m_CurrentState = m_Available = m_StatesNumber = -1;
	m_ReportLog = 0;
	m_States = 0;
	SetTitle("Untitled");
}

XAutomat::XAutomat(int num)
{
	m_BeginLogging = m_TransLogging = m_EndLogging = 0;
	
	m_CurrentState = 0;
	m_StatesNumber = 0;
	m_Available = num;
	m_States = new XState*[m_Available]; 
	m_ReportLog = 0;
	SetTitle("Untitled");
}

XAutomat::XAutomat(const char* title, int num)
{
	m_BeginLogging = m_TransLogging = m_EndLogging = 1;
	
	m_CurrentState = 0;
	m_StatesNumber = 0;
	m_Available = num;
	m_States = new XState*[m_Available]; 
	m_Title = 0;
	SetTitle(title);
	m_ReportLog = 0;
}

XAutomat::~XAutomat()
{
	if (m_Title)
	{
		delete [] m_Title;
	}
	if (m_States)
	{
		delete [] m_States;
	}
}

void XAutomat::Automat(const XEvent &e)
{
	int OldState = m_CurrentState;
	
	LogBegin(m_CurrentState, e.m_Number);
	m_CurrentState = m_States[m_CurrentState]->ChangeState(e);

	if (OldState == m_CurrentState)
	{
		LogEnd(m_CurrentState, e.m_Number);
		return;
	}

	LogTrans(OldState, m_CurrentState);

	if ((m_CurrentState >= m_StatesNumber) || (m_CurrentState < 0))
	{
		LogError(m_CurrentState);
	}

	m_States[m_CurrentState]->EnterInfluence();
	LogEnd(m_CurrentState, e.m_Number);
}

void XAutomat::SetTitle(const char* title)
{
	if (m_Title)
	{
		delete [] m_Title;
	}
	int len = strlen(title);
	m_Title = new char[len + 1];
	strcpy(m_Title, title);
}

void XAutomat::ReceiveEvent(const XEvent &e)
{
	Automat(e);
}

int XAutomat::GetCurrentState(void)
{
	return m_CurrentState;
}

int XAutomat::GetStatesNumber(void)
{
	return m_StatesNumber;
}

XState* XAutomat::GetState(int num)
{
	if ((num < m_StatesNumber) & (num > -1))
	{
		return m_States[num];
	}
	else 
	{
		return 0;
	}
}

void XAutomat::AddState(XState *s)
{
	if (m_Available == m_StatesNumber)
	{
		m_Available *= 2;
        XState** tmp = new XState*[m_Available];
		memcpy((char*)tmp, ((const char*)m_States), sizeof(XState*) *  m_StatesNumber);
		delete [] m_States;
		m_States = tmp;
	}
	m_States[m_StatesNumber] = s;
	m_StatesNumber++;
}

bool XAutomat::InsertState(XState* s, int num)
{
	if (num >= (m_StatesNumber - 1))
	{
		return 0;
	}
	m_States[num] = s;
	return 1;
}

void XAutomat::InitLog(ofstream* fOut, bool begin, bool trans, bool end)
{
	m_ReportLog = fOut;
	m_BeginLogging = begin;
	m_TransLogging = trans;
	m_EndLogging = end;
	(*m_ReportLog) << m_Title << " was initialized" << endl;
}

void XAutomat::TerminateLog(void)
{
	(*m_ReportLog) << m_Title << " was terminated" << endl;
}

void XAutomat::LogBegin(int state, int event)
{
	if (!m_BeginLogging)
	{
		return;
	}
	(*m_ReportLog).width(8);
	(*m_ReportLog).setf(ios::left);
	(*m_ReportLog).fill(' ');
	(*m_ReportLog) << m_Title;
	(*m_ReportLog).width(4);
	(*m_ReportLog).setf(ios::right);
	(*m_ReportLog).fill(' ');	
	(*m_ReportLog) << state << " ( ";
	(*m_ReportLog).unsetf(ios::right);
	(*m_ReportLog).width(20);
	(*m_ReportLog).fill(' ');
	(*m_ReportLog) << m_States[state]->GetTitle() << ")  <- e" << event << endl;
}

void XAutomat::LogTrans(int OldState, int NewState)
{
	if (!m_TransLogging)
	{
		return;
	}	
	(*m_ReportLog).width(8);
	(*m_ReportLog).setf(ios::left);
	(*m_ReportLog).fill(' ');
	(*m_ReportLog) << m_Title;
	(*m_ReportLog).width(4);
	(*m_ReportLog).setf(ios::right);
	(*m_ReportLog).fill(' ');	
	(*m_ReportLog) << OldState << " ( ";
	(*m_ReportLog).unsetf(ios::right);
	(*m_ReportLog).width(20);
	(*m_ReportLog).fill(' ');
	(*m_ReportLog) << m_States[OldState]->GetTitle() << ")  -> ";
	(*m_ReportLog).width(4);
	(*m_ReportLog).setf(ios::right);
	(*m_ReportLog).fill(' ');	
	(*m_ReportLog) << NewState << " ( ";
	(*m_ReportLog).unsetf(ios::right);
	(*m_ReportLog).width(20);
	(*m_ReportLog).fill(' ');
	(*m_ReportLog) << m_States[NewState]->GetTitle() << ")"<< endl;
}

void XAutomat::LogEnd(int state, int event)
{
	if (!m_EndLogging)
	{
		return;
	}	
	(*m_ReportLog).width(8);
	(*m_ReportLog).setf(ios::left);
	(*m_ReportLog).fill(' ');
	(*m_ReportLog) << m_Title;
	(*m_ReportLog).width(4);
	(*m_ReportLog).setf(ios::right);
	(*m_ReportLog).fill(' ');	
	(*m_ReportLog) << state << " ( ";
	(*m_ReportLog).unsetf(ios::right);
	(*m_ReportLog).width(20);
	(*m_ReportLog).fill(' ');
	(*m_ReportLog) << m_States[state]->GetTitle() << ")  finished (e" << event << ")" << endl;	
	
}

void XAutomat::LogError(int state)
{
	(*m_ReportLog) <<  m_Title << " have changed state to unknown (" << state << ")" << endl;
}