
/*==================================[ REDEFINITION DEFENCE ]==================*/
#ifndef		AUTOMAT_H
#define		AUTOMAT_H


#pragma once

#include <fstream>
using namespace std;

/*----------------------------------[ XEvent ]---------------------------*/
class XEvent
{
public:

/*================================[ PUBLIC CLASS ATRIBUTES ]==================*/

	int		m_Number;
	void*	m_Info;

/*==================================[ PUBLIC    CLASS METHODS ]===============*/

	XEvent();
	XEvent(int n);
	~XEvent();
};

/*----------------------------------[ XState ]---------------------------*/

class XState
{	
/*================================[ PRIVATE CLASS ATRIBUTES ]==================*/

private:

	char* m_Title;

/*==================================[ PUBLIC    CLASS METHODS ]===============*/

public:

	XState();
	XState(const char* title);

    void SetTitle(const char* title);
	char* GetTitle();

	virtual int ChangeState(const XEvent& e) = 0;
	virtual void EnterInfluence() = 0;
	~XState();
};

/*----------------------------------[ XAutomat ]---------------------------*/

class XAutomat
{
/*================================[ PRIVATE CLASS ATRIBUTES ]==================*/

private:

	char*		m_Title;			// automat's title

	XState**	m_States;			// array of states
	int			m_CurrentState;		// current state of automat
	int			m_StatesNumber;		// number of states in automat
	int			m_Available;		// number reserved states

	ofstream*	m_ReportLog;		// pointer to log file
	bool		m_BeginLogging;		// if true activate LogBegin   
	bool		m_TransLogging;		// if true activate LogTrans
	bool		m_EndLogging;		// if true activate LogEnd


/*==================================[ PRIVATE   CLASS METHODS ]===============*/

private:

	void Automat(const XEvent& e);		// automat procedure

	void LogBegin(int state, int event);
	void LogTrans(int OldState, int NewState);
	void LogEnd(int state, int event);

	void LogError(int state);

/*==================================[ PUBLIC    CLASS METHODS ]===============*/

public:

	XAutomat();
	XAutomat(int num);
	XAutomat(const char* title, int num);

	~XAutomat();

	void SetTitle(const char* title);
	
	void ReceiveEvent(const XEvent &e);	// interface function for other objects

	int GetCurrentState(void);

	int GetStatesNumber(void);

	XState* GetState(int num);

	void AddState(XState* e);		// add state to the end of automat

	// insert state into defined place 
	bool InsertState(XState* e, int num);

	// initalize logging
	void InitLog(ofstream* fOut, bool begin, bool trans, bool end); 

	// final automat's word 
	void TerminateLog(void);

};

/*==================================[ END REDEFINITION DEFENCE ]==============*/
#endif		AUTOMAT_H
